#include <fpp.h>

#if STDC
static void
print_lines(FILE * outfile, const char *lines[])
#else /* NOT STDC */
static void
print_lines(outfile, lines)
FILE * outfile;
const char *lines[];
#endif /* STDC */
{
    /*******************************************************************
      Print the array of lines[] on outfile, supplying a newline after
      each.
    *******************************************************************/

    size_t k;

    for (k = 0; lines[k] != (const char *)NULL; ++k)
	(void)fprintf(outfile, "%s\n", lines[k]);
    FFLUSH(outfile);
}


static void
opt_author(VOID_ARG)
{
    static const char *lines[] = {
	"Author:",
	"\tNelson H. F. Beebe",
	"\tCenter for Scientific Computing",
	"\tUniversity of Utah",
	"\tDepartment of Mathematics, 322 INSCC",
	"\t155 S 1400 E RM 233",
	"\tSalt Lake City, UT 84112-0090",
	"\tUSA",
	"\tEmail: beebe@math.utah.edu, beebe@acm.org,",
	"\t       beebe@computer.org, beebe@ieee.org (Internet)",
	"\tWWW URL: http://www.math.utah.edu/~beebe",
	"\tTelephone: +1 801 581 5254",
	"\tFAX: +1 801 585 1640, +1 801 581 4148",
	(const char *)NULL,
    };

    print_lines(stderr, lines);
}


static void
opt_copyright(VOID_ARG)
{
    static const char *lines[] = {
	"########################################################################",
	"########################################################################",
	"########################################################################",
	"###                                                                  ###",
	"### fpp: cpp-like reversible preprocessor filter for Fortran and     ###",
	"### SFTRAN3 code                                                     ###",
	"###                                                                  ###",
	"###        Copyright (C) 1990, 1993, 2001 Nelson H. F. Beebe         ###",
	"###                                                                  ###",
	"### This program is covered by the GNU General Public License (GPL), ###",
	"### version 2 or later, available as the file COPYING in the program ###",
	"### source distribution, and on the Internet at                      ###",
	"###                                                                  ###",
	"###               ftp://ftp.gnu.org/gnu/GPL                          ###",
	"###                                                                  ###",
	"###               http://www.gnu.org/copyleft/gpl.html               ###",
	"###                                                                  ###",
	"### This program is free software; you can redistribute it and/or    ###",
	"### modify it under the terms of the GNU General Public License as   ###",
	"### published by the Free Software Foundation; either version 2 of   ###",
	"### the License, or (at your option) any later version.              ###",
	"###                                                                  ###",
	"### This program is distributed in the hope that it will be useful,  ###",
	"### but WITHOUT ANY WARRANTY; without even the implied warranty of   ###",
	"### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    ###",
	"### GNU General Public License for more details.                     ###",
	"###                                                                  ###",
	"### You should have received a copy of the GNU General Public        ###",
	"### License along with this program; if not, write to the Free       ###",
	"### Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   ###",
	"### MA 02111-1307 USA.                                               ###",
	"########################################################################",
	"########################################################################",
	"########################################################################",
	(const char *)NULL
    };

    print_lines(stderr, lines);
}


static void
opt_help(VOID_ARG)
{
    static const char *lines[] = {
	"fpp [ -? ] [ -author ] [ -copyright ] [ -Dname ]",
	"    [ -Dname=value ] [ -debug ] [ -fixed ] [ -free ]",
	"    [ -help ] [ -Uname ] [ -version ] [ name1=value1 ] ...",
	"    [ namek=valuek ] [ -- ] [ infile(s) ]",
	(const char *)NULL
    };

    print_lines(stderr, lines);
}


#if STDC
int
doarg(const char *s)
#else /* NOT STDC */
int
doarg(s)		/* return 0 if arg, 1 if last arg, 2 if file  */
const char *s;
#endif /* STDC */
{
    if ((s[0] == '-') && (s[1] == '-') && (s[2] != '\0'))
	s++;		/* handle GNU/POSIX --option prefix */
    if ((s[0] == '+') || (s[0] == '-'))
    {			/* [-+][DU]name or [-+][DU]name=value */
	size_t n;
	n = strlen(s+1);
	n = (size_t)MAX(n,1);
	if (strncmp(s+1,"?",n) == 0)
	{
	    opt_help();
	    exit(EXIT_SUCCESS);
	}
	else if (strncmp(s+1,"author",n) == 0)
	{
	    opt_author();
	    exit(EXIT_SUCCESS);
	}
	else if (strncmp(s+1,"copyright",n) == 0)
	{
	    opt_copyright();
	    exit(EXIT_SUCCESS);
	}
	else if (strncmp(s+1,"debug",n) == 0)
	{
	    fpp_debug = 1;
	    return(0);
	}
	else if (strncmp(s+1,"fixed",(size_t)MAX(n,2)) == 0)
	{
	    g_src_form = SF_FIXED;
	    set_src_form("");
	    return(0);
	}
	else if (strncmp(s+1,"free",(size_t)MAX(n,2)) == 0)
	{
	    g_src_form = SF_FREE;
	    set_src_form("");
	    return(0);
	}
	else if (strncmp(s+1,"help",n) == 0)
	{
	    opt_help();
	    exit(EXIT_SUCCESS);
	}
	else if (strncmp(s+1,"version",n) == 0)
	{
	    (void)fprintf(stderr, "%s\n", version());
	    FFLUSH(stderr);
	    exit(EXIT_SUCCESS);
	}
	else
	{
	    switch (s[1])
	    {
	    case '-':
		return(1);

	    case 'D':
		dodef(s+2);
		break;

	    case 'U':
		doundef(s+2);
		break;

	    default:
		warning("Unrecognized command-line option [", s, "]");
		break;
	    }
	    return (0);
	}
    }
    else if (strchr(s,'=') != (char*)NULL) /* name=value */
    {
	dodef(s);
	return (0);
    }
    else				/* must be filename */
	return (2);
    return (0);				/* to keep compilers happy */
}
